package Net::Checkpoint::Management::v1::Role::REST::Client;
$Net::Checkpoint::Management::v1::Role::REST::Client::VERSION = '0.001009';
# ABSTRACT: Checkpoint Management API version 1.x client
use Moo::Role;

with 'Role::REST::Client';

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Net::Checkpoint::Management::v1::Role::REST::Client - Checkpoint Management API version 1.x client

=head1 VERSION

version 0.001009

=head1 AUTHOR

Alexander Hartmaier <abraxxa@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2020 by Alexander Hartmaier.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
